/*
	Dash Launch 2 main core code
	[cOz]
*/

#include <xtl.h>
#include <new.h>
#include <stdio.h>
#include <string.h>
#include <ppcintrinsics.h>
#include "kernel.h"
#include "launch.h"
#include "utility.h"
#include "SimpleIni.h"
#include "privs.h"
#include "../_common.h"
// #include "cprintf.h"

#define  _HAS_EXCEPTIONS  0

BOOL g_FirstRun = TRUE;
BOOL g_LaunchDashNoExcept = TRUE;
DWORD g_status[2]; // 0 = return status, 1 = completion signal
DWORD g_Region = 0;
PDWORD g_patches;
DWORD g_PatchState = PATCHSTATE_WAITING;
DWORD g_Protection = PROTECT_OFF; // 1 = on, 0 = off
keydata launch[10];
CSimpleIniA g_ini(1, 0, 0);
DWORD g_options;

#define getOpt(x)	((g_options&x)!=0)
#define setOpt(x)	(g_options = g_options|x)
#define unSetOpt(x) (g_options = g_options&~x)

// version and launch helper data structure
extern "C" ldata ldat = {
	LAUNCH_DATA_ID,	// DWORD ID;
	LHELPER_CON,	// DWORD ltype;
	"",				// char link[MAX_PATH];
	"",				// char dev[MAX_PATH];
	VER_MAJ,		// USHORT versionMaj;
	VER_MIN,		// USHORT versionMin;
	TARGET_KERNEL	// USHORT targetKernel;
};

WORD getButtons(DWORD reps)
{
	WORD buttons = 0;
	DWORD user = 0, i;
	PDWORD devcontx;
	DWORD packet;
	XINPUT_GAMEPAD Gamepad;

	for(i=0; i < reps; i++)
	{
		if(XamUserGetDeviceContext(user, 0, &devcontx) >= 0)
		{
			if(XInputdReadState(devcontx, &packet, &Gamepad) >= 0)
			{
				if(Gamepad.wButtons != 0)
				{
					buttons = Gamepad.wButtons;
					i = 50;
				}
				else
					Sleep(35);
			}
			else
				Sleep(35);
		}
	}
	return buttons;
}

DWORD mountToPath(DWORD mountItem)
{
	DWORD sta;
	if(mountItem >= MOUNT_MAX_ITEMS)
		return 0;
	if(mountItem == MOUNT_HDD)
	{
		if((XboxHardwareInfo->Flags && XBOX_HW_FLAG_HDD) == 0)
			return 0;
	}
	sta = (MountPath(MOUNT_NAME, mountPath[mountItem], FALSE) == S_OK);
	DbgPrint("launch.xex: mount %s to %s sta:%s\n", mountPath[mountItem], MOUNT_NAME, (sta == 0 ? "Fail" : "Success"));
	return sta;
}

BOOL checkLaunchData(const char * arg1, const char * arg2, const char * arg3, DWORD flags)
{
	g_LaunchDashNoExcept = TRUE;
	// when straight dash loads, all three args are NULL
	// when a natal or other games load, flags is 0x4 arg1 is a pointer, arg2 if a con is used, other 0x0
	// launching a normal game or dvd video, flags set to 0x12, other 0x0
	// eject game, arg1 set and flags 0x2000, other 0x0
	// eject dvd vid all args 0x0
	// when exiting arcade to dashboard, flags is set to 0x104, other 0x0
	//DbgPrint("ld: %08x %08x %08x %08x\n", arg1, arg2, arg3, flags);
	//if(arg1 != NULL)
	//	DbgPrint("arg1: %s %02x %02x %02x\n", arg1, arg1[0], arg1[1], arg1[2]);
	
	// media center case
	//ld: fdd53000 00699000 92031004 00000001
	//arg1: XEX2 58 45 58
	if((flags == 1)&&(arg3 != NULL))
	{
		g_LaunchDashNoExcept = FALSE;
	}
	// check for kinect game launches which call on dash.xex
	else if((flags == 0x4)&&(arg1 != NULL))
	{
		if(arg1[0] != 0) // when homebrew exits it went to NXE, arg1[0]=0x0
			g_LaunchDashNoExcept = FALSE;
	}
	else if(arg2 != NULL)
	{
		if(strnicmp(arg2, "XSYSLAUNCH", strlen("XSYSLAUNCH")) == 0)
			g_LaunchDashNoExcept = FALSE;
		//DbgPrint("arg2: %s\n", arg2);
	}
	else if((arg1 == NULL)&&(arg3 == NULL)) // &&(arg2 == NULL)
	{
		// when dvd player loads a dvd, flags is 0x12 and 3 args are NULL
		if(flags == 0x12)
		{
			g_LaunchDashNoExcept = FALSE;
		}
		else
		{
			// arcade exit
			if((flags == 0x104)&&(getOpt(OPT_XBLAEXIT)))
			{
				g_LaunchDashNoExcept = FALSE;
			}
			else
			{
				DWORD dwLaunchDataSize = 0;
				DWORD dwStatus = XGetLaunchDataSize(&dwLaunchDataSize);

				if(dwStatus == ERROR_SUCCESS)
				{
					if(dwLaunchDataSize != 0)
					{
						BYTE* pLaunchData = new(std::nothrow) BYTE[dwLaunchDataSize];
						if(pLaunchData != NULL)
						{
							dwStatus = XGetLaunchData(pLaunchData, dwLaunchDataSize);
							if(dwStatus == ERROR_SUCCESS)
							{
								//DbgPrint("ldata sz: 0x%x\n", dwLaunchDataSize);
								//DWORD i;
								//for(i = 0; i < dwLaunchDataSize; i++)
								//{
								//	if(pLaunchData[i] != 0x0)
								//		DbgPrint("i:%x %02x\n", i, pLaunchData[i]);
								//}
								// when dash loads, it sets byte 0x7 of launch data for the specific mode/screen to return to
								// byte 0x4 of launch data is set to 0x10 when returning from dvd to dash
								if(pLaunchData[7] != 0x0)
								{
									if(!getOpt(OPT_NOSYSEXIT))
										g_LaunchDashNoExcept = FALSE;
								}
								else if(pLaunchData[4] == 0x10)
								{
									// dvd video exit
									if((getOpt(OPT_DVDEXIT))&&(!getOpt(OPT_NOSYSEXIT)))
										g_LaunchDashNoExcept = FALSE;
								}
								dwStatus = XSetLaunchData(pLaunchData, dwLaunchDataSize);
							}
							delete[] pLaunchData;
						}
					}
				}
			}
		}
	}
	return g_LaunchDashNoExcept;
}

VOID __declspec(naked) XamLoaderLaunchTitleExHook(const char * arg1, const char * arg2, const char * arg3 , UINT32 flags)
{
	__asm{
		mflr    r12
		stw     r12, -8(sp)
		mr      r31, r3
		mr      r27, r4
		mr      r24, r5
		stw		r6, -0xC(sp)
		stwu    sp, -0x10(sp)
		bl		checkLaunchData
		addi    sp, sp, 0x10
		lwz		r6, -0xC(sp)
		li      r5, 0x314
		lwz     r12, -8(sp)
		mtlr    r12
		blr
	}
}

char* enumButtons(SHORT pad, char* xex)
{
	char* xexname = xex;
	DWORD item = 0;
	if((pad == 0) || (pad == 0xFFFF)) // no controller or no button pressed
		item = DEFAULT;
	else if(pad & XINPUT_GAMEPAD_A)
		item = BUT_A;
	else if(pad & XINPUT_GAMEPAD_B)
		item = BUT_B;
	else if(pad & XINPUT_GAMEPAD_X)
		item = BUT_X;
	else if(pad & XINPUT_GAMEPAD_Y)
		item = BUT_Y;
	else if(pad & XINPUT_GAMEPAD_START)
		item = START;
	else if(pad & XINPUT_GAMEPAD_BACK)
		item = BACK;
	else if(pad & XINPUT_GAMEPAD_LEFT_SHOULDER)
		item = LBUMP;
	// in case user was holding an unconfigured button
	if(launch[item].launchpath[0] == 0 || launch[item].flags == INVALID_PATH)
		item = DEFAULT;
	if(launch[item].launchpath[0] != 0)
	{
		if(mountToPath(launch[item].dev))
		{
			HANDLE file = INVALID_HANDLE_VALUE;
			while(file == INVALID_HANDLE_VALUE)
			{
				file = CreateFile(launch[item].launchpath,GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
			}
			CloseHandle(file);
			if(launch[item].flags == CON_PATH)
			{
				ldat.ltype = LHELPER_CON;
				strcpy_s(ldat.link, MAX_PATH, USER_PATH);
				strcat_s(ldat.link, MAX_PATH, launch[item].launchpath);
				strcpy_s(ldat.dev, MAX_PATH, mountPath[launch[item].dev]);
				xexname = helper;
			}
			else if(launch[item].flags == XEX_PATH)
			{
				ldat.ltype = LHELPER_XEX;
				strcpy_s(ldat.link, MAX_PATH, launch[item].launchpath);
				strcpy_s(ldat.dev, MAX_PATH, mountPath[launch[item].dev]);
				xexname = helper;
			}
		}
	}
	return xexname;
}

VOID relaunchThread(void)
{
	Sleep(500);
	MountPath(MOUNT_NAME, mountPath[MOUNT_FLASH], FALSE);
	XLaunchNewImage("dlaunch:\\lhelper.xex", 0);
}

UINT32 (*XNotifyBroadcast)(UINT32, PVOID);
UINT32 XNotifyBroadcastNew(PVOID data)
{
	UINT32 ret = 0xB0BB00B5;
	UINT32 cmd = 0x80040016;
	if(getOpt(OPT_NXEMINI))
	{
		if(checkLaunchData(NULL, NULL, NULL, 0))
		{
			ret = XNotifyBroadcast(cmd, data); // let the miniblades close
			if(data == NULL) // anything else and it's sending a message to dash, like login
			{
				WORD but = 0x8000;
				while(but == 0x8000)
					but = getButtons(10);
				if((but & XINPUT_GAMEPAD_RIGHT_SHOULDER) == 0)
				{
					char* xexname = enumButtons(but, NULL);
					if(xexname != NULL)
					{
						// we need to launch helper now...
						HANDLE hThread;
						DWORD dwThreadId;
						hThread = CreateThread( 0, 0, (LPTHREAD_START_ROUTINE)relaunchThread, 0, 0, &dwThreadId );
						CloseHandle(hThread);
					}
				}
			}
		}
	}
	if(ret == 0xB0BB00B5)
		ret = XNotifyBroadcast(cmd, data);
	return ret;
}

UINT32 __declspec(naked) XamLoaderRebootNotify()
{
	__asm{
		mflr    r12				// save link register to phony stack frame
			stw     r12, -8(sp)
			stwu    sp, -0x10(sp)

			slwi    r11, r31, 16
			//lis     r3, -0x7FFC // 0x80040016
			or      r3, r11, r30 //r4, r11, r30
			//ori     r3, r3, 0x16 // 0x80040016
			bl		XNotifyBroadcastNew //bl      XNotifyBroadcast

			addi    sp, sp, 0x10	// destroy artificial stack frame
			lwz     r12, -8(sp)		
			addi	r12, r12, 4		// skipping the built in xnotify
			mtlr    r12
			blr    
	}
}

char* loadPrep(const char* xex)
{
	char* xexname = (char*)xex;
	WORD pad;
	unSetOpt(OPT_REGION);
	if(strncmp(xex, XBOX_XEX, strlen(XBOX_XEX)) == 0)
	{
		HvxSetState(SET_PROT_ON);
		g_Protection = PROTECT_ON;
		__dcbst(0, &g_Protection);
		__sync();
	}
	else if(strcmp(xex, DASH_XEX) == 0)
	{
		if(g_Protection)
		{
			HvxSetState(SET_PROT_OFF);
			g_Protection = PROTECT_OFF;
			__dcbst(0, &g_Protection);
			__sync();
		}
		if(g_LaunchDashNoExcept)
		{
			if(g_FirstRun)
			{
				firstRunTasks();
				pad = getButtons(30);
			}
			else
				pad = getButtons(10);
			if((pad & XINPUT_GAMEPAD_RIGHT_SHOULDER) == 0)
			{
				xexname = enumButtons(pad, xexname);
			}
		}
	}
	else if(g_Region != 0)
	{
		pad = getButtons(5);
		if((pad & XINPUT_GAMEPAD_RIGHT_SHOULDER))
		{
			setOpt(OPT_REGION);
		}
	}
	DbgPrint("launching '%s'\n", xexname);
	return xexname;
}

VOID __declspec(naked) LoaderPrepare(UINT32 argR3, const char * xex, UINT32 argR5 , PHANDLE handle, UINT32 typeinfo, UINT32 ver, UINT32 argR9, UINT32 argR10, UINT32 argSt1)
{
	__asm{

			li      r30, 0			// instructions replaced to get here
			mr      r29, r3			//	argR3
			//mr      r31, r4		//	xex path
			mr      r27, r5
			mr      r24, r6			//	phandle
			//li		r25, 0
			ld      r25, 0x390(r29)
			mr      r23, r30 //r7			//	typeinfo
			mr      r26, r10
			mr      r28, r30
			stw     r30, 0x10C(sp)	//	version

			mflr    r12				// save link register to phony stack frame
			stw     r12, -8(sp)
			stwu    sp, -0x10(sp)

			mr		r30, r9			// save the reg for later

			mr		r3, r4			// give dash launch the xex path to play with
			bl      loadPrep
			mr		r31, r3			// replace the xex path on return

			mr      r9, r30			// return it from where we saved it
			li      r30, 0			// put zero back in to restore for rest of function

			addi    sp, sp, 0x10	// destroy artificial stack frame
			lwz     r12, -8(sp)		//	return to 81877CF4
			addi	r12, r12, 0x18	//	we want it to go back to 81877D0C
			mtlr    r12
			blr
	}
}

DWORD XGetGameRegionNew(VOID)
{
	if(getOpt(OPT_REGION))
	{
		return g_Region;
	}
	else
	{
		// we can call the original function because it was linked before xam hook was put in place
		return XGetGameRegion();
	}
}

BOOL XamCheckExecPriv(DWORD priv)
{
	BOOL ret = XexCheckExecutablePrivilege(priv);
	//DbgPrint("priv: %d ret %d\n", priv, ret);
	if(priv == PRIV_INSECURE_SOCKS)
	{
		if(getOpt(OPT_SOCKPATCH))
			ret = TRUE;
	}
	else if(priv == PRIV_AP25_MEDIA) // remove AP25 flags, we don't need no steenkin AP25
		ret = FALSE;
	//else if(priv == PRIV_TITLE_INSTALL_INCOMPAT)
	//	ret = FALSE;
	return ret;
}

DWORD parseIni(char* path)
{
	HANDLE file = CreateFile(path,GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(file == INVALID_HANDLE_VALUE)
		return 0;
	CloseHandle(file);
	SI_Error rc = g_ini.LoadFile(path);
	if (rc < 0)
	{
		return 0;
	}
	return 1;
}

VOID loadPlugins(VOID)
{
	const char * fileName;
	DWORD sta = 0, i, j;
	char cat[] = "plugin1";
	for(i = 0; i < 5; i++)
	{
		cat[6] = (i + 0x31) & 0xFF;
		fileName = g_ini.GetValue(PLUGIN_KEY, cat, NULL);
		if((fileName != NULL)&&(fileName[0] != 0))
		{
			char plugpath[MAX_PATH];
			DWORD devNum = 0xF, testNum = 0;
			
			// figure out which mount point was used
			for(j = 0; j < DEVS_MAX; j++)
			{
				if(strnicmp(fileName, numDevs[j].mountPoint, (numDevs[j].stlen+1)) == 0)
				{
					devNum = numDevs[j].startdev;
					testNum = numDevs[j].numdev;
					strcpy_s(plugpath, MAX_PATH, "dlaunch");
					strcat_s(plugpath, MAX_PATH, (fileName+numDevs[j].stlen));
				}
			}
			// check if the file exists
			if(devNum != 0xF)
			{
				HANDLE file;
				for(j = 0; j < testNum; j++)
				{
					mountToPath(devNum);
					file = CreateFile(plugpath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
					if(file != INVALID_HANDLE_VALUE)
					{
						CloseHandle(file);
						sta = XexLoadImage(plugpath, 8, 0, NULL);
						j = testNum;
					}
					else
						devNum++;
				}
			}
		}
	}
}

VOID doPatches(VOID)
{
	u32 * addr;
	u32 len;
	DWORD i=0, j;
	BOOL doPatches = TRUE;

	if(g_patches != NULL)
	{
		while(doPatches)
		{
			if((g_patches[i] == 0x0)||(g_patches[i] == 0xFFFFFFFF)) // invalid address, end of file marker
			{
				doPatches = FALSE;
			}
			else
			{
				addr = (u32*)(g_patches[i]);
				i++;
				len = g_patches[i];
				i++;
				for(j = 0; j < len; j++)
				{
					addr[j] = g_patches[i];
					i++;
				}
				__dcbst(0, addr);
				__sync();
			}
		}
		delete[] g_patches;
	}
}

DWORD loadPatches(char* path)
{
	HANDLE file = CreateFile(path,GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(file == INVALID_HANDLE_VALUE)
		return 0;
	g_patches = new(std::nothrow) DWORD[MAX_PATCH_SZ];
	if(g_patches != NULL)
	{
		DWORD bRead = 0;
		ZeroMemory(g_patches, MAX_PATCH_SZ*4);
		ReadFile(file, g_patches, 0x1000*4, &bRead, NULL);
		CloseHandle(file);

		if(bRead < 12) // not enough data for a patch
			return 0;
		if((bRead %4) != 0) // unaligned, not valid patch file
			return 0;
		return 1;
	}
	else
	{
		return 0;
	}
}

VOID checkType(DWORD item)
{
	HANDLE file;
	DWORD bRead, i;
	DWORD type = INVALID_PATH, devNum = 0xF, testnum = 0;
	char tbuf[0x5]; // stow the first bytes of the file to decide if xex or con
	const char* lpath;
	lpath = g_ini.GetValue(BUTTON_KEY, butNames[item], NULL);

	if(lpath != NULL)
	{
		if(lpath[0] != 0)
		{
			//DbgPrint("\n\nparsing: %s\n", butNames[item]);
			// figure out which mount point was used
			for(i = 0; i < DEVS_MAX; i++)
			{

				if(strnicmp(lpath, numDevs[i].mountPoint, (numDevs[i].stlen+1)) == 0)
				{
					devNum = numDevs[i].startdev;
					testnum = numDevs[i].numdev;
					strcpy_s(launch[item].launchpath, MAX_PATH, "dlaunch");
					strcat_s(launch[item].launchpath, MAX_PATH, (lpath+numDevs[i].stlen));
					//DbgPrint("lpath: %s\n", lpath);
					//DbgPrint("Mount: %s\n", launch[item].launchpath);
				}
			}
			// check if the file exists and what type of file it is
			if(devNum != 0xF)
			{
				for(i=0; i < testnum; i++)
				{
					mountToPath(devNum);
					file = CreateFile(launch[item].launchpath, GENERIC_READ, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
					if(file != INVALID_HANDLE_VALUE)
					{
						//DbgPrint("FOUND! %s\n", launch[item].launchpath);
						ReadFile(file, tbuf, 0x4, &bRead, NULL);
						CloseHandle(file);
						tbuf[4] = 0; // make sure it's null terminated
						if(strcmp(TYPE_STR_LIVE, tbuf) == 0)
						{
							type = CON_PATH;
							i = testnum;
						}
						else if(strcmp(TYPE_STR_XEX, tbuf) == 0)
						{
							type = XEX_PATH;
							i = testnum;
						}
						else
							devNum++;
					}
					else
						devNum++;
				}
			}
		}
	}
	if(type == INVALID_PATH)
	{
		launch[item].flags = type;
		launch[item].dev = 0xF;
	}
	else
	{
		launch[item].flags = type;
		launch[item].dev = devNum;
	}
	return;
}

VOID firstRunTasks(VOID)
{
	DWORD patchLoaded = 0, configLoaded = 0, i;
	char tmp[MAX_PATH];
	g_options = 0;

	// look for patches and ini on usb/hdd/flash in that order
	for(i=0; i < (MOUNT_FLASH+1); i++)
	{ 
		if((!patchLoaded)||(!configLoaded))
		{
			if(mountToPath(i))
			{
				if(!patchLoaded)
				{
					strcpy_s(tmp, MAX_PATH, MOUNT_NAME);
					strcat_s(tmp, MAX_PATH, PATCH_NAME);
					patchLoaded = loadPatches(tmp);
				}
				if(!configLoaded)
				{
					strcpy_s(tmp, MAX_PATH, MOUNT_NAME);
					strcat_s(tmp, MAX_PATH, INI_NAME);
					configLoaded = parseIni(tmp);
				}
			}
		}
	}

	// check to see if there was patchdata and execute patches if there were
	if(patchLoaded != 0)
		doPatches();

	// parse and unload config ini if it was found
	if(configLoaded)
	{
		for(i=0; i<MAX_NUM_BUTTONS; i++)
		{
			checkType(i);
		}
		// sets NXE miniblade option
		if(g_ini.GetBoolValue(SETTING_KEY, "nxemini", TRUE))
			setOpt(OPT_NXEMINI);

		if(g_ini.GetBoolValue(SETTING_KEY, "xblaexitdash", FALSE))
			setOpt(OPT_XBLAEXIT);

		if(g_ini.GetBoolValue(SETTING_KEY, "dvdexitdash", FALSE))
			setOpt(OPT_DVDEXIT);

		if(g_ini.GetBoolValue(SETTING_KEY, "nosysexit", FALSE))
			setOpt(OPT_NOSYSEXIT);

		// handles patching the fatal exception functionality of kernel
		if(!(g_ini.GetBoolValue(SETTING_KEY, "fatalfreeze")))
		{
			PDWORD ptr = (PDWORD)HalHaltSystemPatch;
			if(g_ini.GetBoolValue(SETTING_KEY, "fatalreboot")) // they wanted reboot instead of poweroff
				ptr[0] = 0x38600002; // reboot li r3, 2
			else
				ptr[0] = 0x38600005; // poweroff li r3, 5
			ptr[1] = 0x4BFF6BFC; // b HalReturnToFirmware (f:0x711A8 t: 0x67da0 = -0x9408 = FF6BFC
		}

		// removes ping timeouts from system link games
		if(g_ini.GetBoolValue(SETTING_KEY, "pingpatch", FALSE))
		{
			// patch by FBDev
			PDWORD ptr = (PDWORD)PingLimitPatch;
			ptr[0] = 0x60000000;
			__dcbst(0, ptr);
			__sync();
		}

		// does essentially what yaris swap does
		if(g_ini.GetBoolValue(SETTING_KEY, "contpatch", FALSE))
		{
			// patch by mojobojo @ xboxhacker
			PDWORD ptr = (PDWORD)XContentGetLicenseMask;
			ptr[0] = 0x3960FFFF;
			ptr[1] = 0x91630000;
			ptr[2] = 0x38600000;
			ptr[3] = 0x4E800020;
			__dcbst(0, ptr);
			__sync();
			ptr = (PDWORD)DasboardGameLicenseCheck;
			ptr[0] = 0x3D60FFFF;
			ptr[1] = 0x3B400000;
			ptr[2] = 0x617EFFFF;
			__dcbst(0, ptr);
			__sync();
		}

		// those of you actually looking at the code may use this... thanks again to Jefff! (sockpatch = true)
		// Enables /privilege:6 for all titles
		if(g_ini.GetBoolValue(SETTING_KEY, "sockpatch", FALSE))
		{
			setOpt(OPT_SOCKPATCH);
		}

		if(g_ini.GetBoolValue(SETTING_KEY, "regionspoof", FALSE))
		{
			HookOrd(972, (DWORD)XGetGameRegionNew, XAM_EAT);
			g_Region = g_ini.GetLongValue(SETTING_KEY, "region", 0x7FFF);
		}
		else
			g_Region = 0;

		// last but not least, parse plugins and load them as needed
		loadPlugins();

		g_ini.Reset(); // deallocate everything simpleini used
	}

	g_FirstRun = FALSE;
}

// ** for export to load system dll **
VOID launchSysModThread(char* modPath)
{
	g_status[0] = XexLoadImage(modPath,8,0,NULL);
	g_status[1] = 1;
	//DbgPrint("launch: load returns %x", g_status[0]);
	__dcbst(0, g_status);
	__sync();
}

// ** for export to load system dll **
extern "C" DWORD dlaunchStartSysModule(char* modPath)
{
	HANDLE pthread;
	DWORD pthreadid;
	DWORD sta;
	g_status[0] = 0;
	g_status[1] = 0;
	__dcbst(0, g_status);
	__sync();
	//DbgPrint("launch: attempting to load %s", modPath);
	sta = ExCreateThread(&pthread, 0, &pthreadid, (PVOID) XapiThreadStartup , (LPTHREAD_START_ROUTINE)launchSysModThread, modPath, 0x2);
	XSetThreadProcessor(pthread, 4);
	ResumeThread(pthread);
	CloseHandle(pthread);

	// wait for thread to run it's course
	while(g_status[1] == 0)
	{
		Sleep(100);
	}

	return g_status[0];
}

//DWORD XexLoadExecutableHook(PCHAR xexName, PHANDLE handle, DWORD typeInfo, DWORD ver)
//{
//	DWORD ret;
//	DbgPrint("xam loadexec: %s h:%08x t:%08x v:%08x\n", xexName, handle, typeInfo, ver);
//	ret = XexLoadExecutable(xexName, handle, typeInfo, ver);
//	DbgPrint("LE: return %08x\n", ret);
//	return ret;
//}
//
//DWORD XexLoadImageHook(PCHAR xexName, DWORD typeInfo, DWORD ver, PHANDLE modHandle)
//{
//	DWORD ret;
//	DbgPrint("xam loadimage: %s t:%08x v:%08x h:%08x\n", xexName, typeInfo, ver, modHandle);
//	ret = XexLoadImage(xexName, typeInfo, ver, modHandle);
//	DbgPrint("LI ret: %08x\n", ret);
//	return ret;
//}

BOOL APIENTRY DllMain(HANDLE hInstDLL, DWORD reason, LPVOID lpReserved)
{
	DbgPrint("launch entry!\n");
	if(reason == DLL_PROCESS_ATTACH)
	{
		// using this to catch exit from miniblade while in NXE, first hook because it needs to be done before hud loads up
		patchInJump((PDWORD)PATCH_LOC_XamLoaderRebootToDash, (DWORD)XamLoaderRebootNotify, TRUE, NULL);
		XNotifyBroadcast = (UINT32 (__cdecl *)(UINT32, PVOID))(resolveFunct("xam.xex", XAM_XNOTIFY_BROADCAST_ORD));

		// using this to catch dash.xex and xbox emu loading
		patchInJump((PDWORD)PATCH_LOC_LoaderPrepare, (DWORD)LoaderPrepare, TRUE, NULL);

		// using this to catch launch data to find out if we are returning to dash or not
		// by the time LoaderPrepare fires launchdata has already been dealt with by xam
		patchInJump((PDWORD)PATCH_LOC_LaunchTitleEx, (DWORD)XamLoaderLaunchTitleExHook, TRUE, NULL);

		// to modify privileges more easily...
		patchInJump((PDWORD)PATCH_LOC_XamCheckExecPriv, (DWORD)XamCheckExecPriv, FALSE, NULL);

		// some logging functions
		//patchInJump((PDWORD)PATCH_LOC_XexLoadExecutable, (DWORD)XexLoadExecutableHook, FALSE, NULL);
		//patchInJump((PDWORD)PATCH_LOC_XexLoadImage, (DWORD)XexLoadImageHook, FALSE, NULL);
	}
	return TRUE;
}
