/* protos and defines for (only) launch.c */

#ifndef _LAUNCH_H_
#define _LAUNCH_H_

// for relaunching stuff
char helper[] = "\\SystemRoot\\lhelper.xex";

/* static per firmware patch/hook locations */
#define PATCH_LOC_XexLoadImage			(0x81BC6BE4) // XexLoadImage
#define PATCH_LOC_XexLoadExecutable		(0x81BC7394) // XexLoadExecutable stub
#define PATCH_LOC_LoaderPrepare			(0x81877CE4) // TitleLoaderPrepareLoadExecutableFile
#define PATCH_LOC_LaunchTitleEx			(0x81879E7C) // XamLoaderLaunchTitleEx
#define PATCH_LOC_XamLoaderRebootToDash	(0x8187A46C) // XamLoaderRebootToDash
#define PATCH_LOC_XamCheckExecPriv		(0x81BC6C14) // XexCheckExecutablePrivilege stub

/* static per firmware patch locations */
#define PingLimitPatch					(0x8191D33C)
#define XContentGetLicenseMask			(0x818C1178) // XamContentGetLicenseMask
#define DasboardGameLicenseCheck		(0x8185D5D4) // ContentEvaluateLicense
#define HalHaltSystemPatch				(0x800711A0) // KeBugCheckEx
#define XamSocketOptionCheck			(0x81910118)

// ordinals
#define XAM_XNOTIFY_BROADCAST_ORD		(0x28E) // 654
#define XEX_CHECK_PRIV_ORD				(0x194)

// 12611 export table addresses
#define XAM_EAT		(PHV_IMAGE_EXPORT_TABLE)(0x81BC8684)
#define KERN_EAT	(PHV_IMAGE_EXPORT_TABLE)(0x80060000) // .xedata segment

/* a useful struct template for an import pointer */
#define XBOX_HW_FLAG_HDD		0x00000020 // if this bit is set the box has a hdd

// option bitfields
#define OPT_NXEMINI				0x1
#define OPT_CONTPATCH			0x2
#define OPT_REGION				0x4
#define OPT_DVDEXIT				0x8
#define OPT_XBLAEXIT			0x10
#define OPT_SOCKPATCH			0x20
#define OPT_NOSYSEXIT			0x40

typedef struct {
	const char* mountPoint;
	int startdev;
	int numdev;
	int stlen;
} NumDevEnum;

typedef struct _keydata {
	char launchpath[MAX_PATH];
	DWORD flags;
	DWORD dev;
} keydata;

/* defines, enums and names for the devices launch will be using */
enum{
	DEFAULT = 0,
	BUT_A,
	BUT_B,
	BUT_X,
	BUT_Y,
	START,
	BACK,
	LBUMP,
	MAX_NUM_BUTTONS
};

enum{
	MOUNT_MASS0 = 0,	// raw usb devices
	MOUNT_HDD = 3,		// hard disk
	MOUNT_SFC = 4,		// flash memory unit big block
	MOUNT_FLASH = 5,	// system flash
	MOUNT_MU0 = 6,		// memory units
	MOUNT_USBMU0 = 8,	// usb memory units
	MOUNT_CDROM = 11,	// CD/DVD drive
	MOUNT_MAX_ITEMS
};

const char* mountPath[] = {
	"\\Device\\Mass0\\",
	"\\Device\\Mass1\\",
	"\\Device\\Mass2\\",
	"\\Device\\Harddisk0\\Partition1\\",
	"\\Device\\BuiltInMuSfc\\", // flash memory unit big block
	"\\Device\\Flash\\",
	"\\Device\\Mu0\\",
	"\\Device\\Mu1\\",
	"\\Device\\Mass0PartitionFile\\Storage\\",
	"\\Device\\Mass1PartitionFile\\Storage\\",
	"\\Device\\Mass2PartitionFile\\Storage\\",
	"\\Device\\Cdrom0\\",
};

char* butNames[]={
	"Default",
	"BUT_A",
	"BUT_B",
	"BUT_X",
	"BUT_Y",
	"Start",
	"Back",
	"LBump"
};

#define DEVS_MAX 7
NumDevEnum numDevs[DEVS_MAX] = {
	{"Usb:",		MOUNT_MASS0,	3, (strlen(numDevs[0].mountPoint)-1)},
	{"Hdd:",		MOUNT_HDD,		1, (strlen(numDevs[1].mountPoint)-1)},
	{"FlashMu:",	MOUNT_SFC,		1, (strlen(numDevs[2].mountPoint)-1)}, // flash memory unit big block
	{"Sfc:",		MOUNT_FLASH,	1, (strlen(numDevs[3].mountPoint)-1)},
	{"Mu:",			MOUNT_MU0,		2, (strlen(numDevs[4].mountPoint)-1)},
	{"UsbMu:",		MOUNT_USBMU0,	3, (strlen(numDevs[5].mountPoint)-1)},
	{"Dvd:",		MOUNT_CDROM,	1, (strlen(numDevs[6].mountPoint)-1)},
};

#define INVALID_PATH	0
#define XEX_PATH		1
#define CON_PATH		2
#define BUTTON_KEY		"QuickLaunchButtons"
#define PLUGIN_KEY		"Plugins"
#define SETTING_KEY		"Settings"
#define INI_NAME		"\\launch.ini"
#define PATCH_NAME		"\\kxam.patch"
#define TYPE_STR_LIVE	"LIVE"
#define TYPE_STR_XEX	"XEX2"
#define USER_PATH		"\\??\\"
#define SYSTEM_PATH		"\\System??\\"
#define XBOX_XEX		"\\Device\\Harddisk0\\SystemPartition\\Compatibility\\xbox.xex"
#define DASH_XEX		"\\SystemRoot\\dash.xex"
// max number of u32's that will be allocated for loading kxam.patch
#define MAX_PATCH_SZ	0x1000

/* state defines */
#define PATCHSTATE_WAITING  0
#define PATCHSTATE_RUNNING  1
#define PATCHSTATE_COMPLETE 2

#define PROTECT_OFF		0
#define PROTECT_ON		1

/* Functions */
WORD getButtons(DWORD reps);
DWORD mountToPath(DWORD mountItem);
BOOL checkLaunchData(const char * arg1, const char * arg2, const char * arg3, DWORD flags);
char* enumButtons(SHORT pad, char* xex);
char* loadPrep(const char* xex);
DWORD parseIni(char* path);
VOID loadPlugins(VOID);
VOID doPatches(VOID);
DWORD loadPatches(char* path);
VOID checkType(DWORD item);
VOID firstRunTasks(VOID);
VOID launchSysModThread(char* modPath);


#endif //_LAUNCH_H_
