//--------------------------------------------------------------------------------------
// AtgTasks.h
//
// Common definitions used by TaskQueue class
//
// Xbox Advanced Technology Group.
// Copyright (C) Microsoft Corporation. All rights reserved.
//--------------------------------------------------------------------------------------
#pragma once

namespace ATG
{

//-----------------------------------------------------------------------------
// Name: struct MessageBase
// Desc: MessageBase baseclass for messages associated with tasks
//-----------------------------------------------------------------------------
struct MessageBase
{
    MessageBase()
    {
        m_cRef      = 0;
        m_pContext  = NULL;
    }

    virtual ~MessageBase()
    {
    }

    VOID AddRef()
    {
        m_cRef++;
    }

    VOID Release()
    {
        if( --m_cRef == 0 )
        {
            delete this;
        }
    }

    int     m_cRef;
    VOID*   m_pContext;
};

//-----------------------------------------------------------------------------
// Name: struct TaskData
// Desc: Structure for holding task data, plus some comparison operators
//-----------------------------------------------------------------------------
typedef struct _TaskData
{
    DWORD        dwMessage;
    MessageBase* pMessage;
    VOID*        pMsgDestination;

    // Assignment operator
    _TaskData& operator = ( const _TaskData& ref )
    {
        dwMessage = ref.dwMessage; 
        pMessage = ref.pMessage;
        pMsgDestination = ref.pMsgDestination;

        return  *this;
    }

    // Use dwMessage for comparison operand
    bool operator < ( _TaskData& ref )
    {
        return dwMessage < ref.dwMessage;
    }

    // Use dwMessage for comparison operand
    bool operator > ( _TaskData& ref )
    {
        return dwMessage > ref.dwMessage;
    }

    #ifdef _DEBUG
    void Dump() const
    {
        DebugSpew( "dwMessage: %d; pMessage: %p; pMsgDestination: %p\n", 
                   dwMessage, pMessage, pMsgDestination);       
    }
    #endif

} TaskData;

#define MAX_SEM_COUNT 65535

//-----------------------------------------------------------------------------
// Name: class CritSec, CAutoCritSec
// Desc: Bare-bones critical section classes
//-----------------------------------------------------------------------------
class CritSec
{	
public:
	CritSec() { InitializeCriticalSection( &m_cs );	}
	~CritSec() { DeleteCriticalSection( &m_cs ); }
	inline void Lock() { EnterCriticalSection( &m_cs ); }
	inline void Unlock() { LeaveCriticalSection( &m_cs ); }				
private:
	CRITICAL_SECTION m_cs;
};

class CAutoCritSec
{
public:
    CAutoCritSec( CritSec* pCritSec ) { m_pCritSec = pCritSec; m_pCritSec->Lock(); }
    ~CAutoCritSec() { m_pCritSec->Unlock(); }
protected:
    CritSec* m_pCritSec;
};

}
