//--------------------------------------------------------------------------------------
//--------------------------------------------------------------------------------------
#include <xtl.h>
#include "AtgConsole.h"
#include "AtgUtil.h"
#include "AtgInput.h"
#include "kernel.h"
#include "devio.h"
#include "iniupdate.h"
#include "util.h"
#include "nand.h"
#include "../_common.h"

char launchpath[]="Usb:\\ldr\\default.xex";

ATG::Console console;

void dprintf(const char* s, ...)
{
	va_list argp;
	char temp[512];

	va_start(argp, s);
	vsnprintf_s(temp, 512,512, s, argp);
	va_end(argp);
	console.Format("%s", temp);
	#ifdef DEBUG_MSG
	DbgPrint("%s", temp);
	#endif
}

// this is doing it the hard way, regular CreateFile and WriteFile work so long as you
// write in multiples of 0x4000 bytes until the last file chunk
DWORD writeFileToFlash(BYTE* buffer, char* fileName, DWORD len)
{
	ANSI_STRING nFlash, fFlash;
	HANDLE hFlashDev, hFlashFile;
	OBJECT_ATTRIBUTES atFlash;
	IO_STATUS_BLOCK ioFlash;
	char destStr[MAX_PATH] = "\\Device\\Flash\\";
	DWORD sta, staNN, dwWritten = 0;
	RtlInitAnsiString(&nFlash, "\\Device\\Flash");
	atFlash.RootDirectory = 0;
	atFlash.ObjectName = &nFlash;
	atFlash.Attributes = FILE_ATTRIBUTE_DEVICE;
	sta = NtOpenFile(&hFlashDev, GENERIC_WRITE|GENERIC_READ|SYNCHRONIZE, &atFlash, &ioFlash, OPEN_EXISTING, FILE_SYNCHRONOUS_IO_NONALERT);
	#ifdef DEBUG_MSG
	dprintf("NtOpenFile(flash): %08x\n",sta);
	#endif
	if(sta < 0)
	{
		return sta;
	}
	sta = NtDeviceIoControlFile(hFlashDev, 0, 0, 0, &ioFlash, 0x3C8044, 0, 0, 0, 0); // sync/async?
	#ifdef DEBUG_MSG
	dprintf("NtDeviceIoControlFile 0x3C8044: %08x\n",sta);
	#endif
	if(sta == 0)
	{
		strcat_s(destStr, MAX_PATH, fileName);
		RtlInitAnsiString(&fFlash, destStr);
		atFlash.RootDirectory = 0;
		atFlash.ObjectName = &fFlash;
		atFlash.Attributes = FILE_ATTRIBUTE_DEVICE;
		sta = NtCreateFile(&hFlashFile,GENERIC_WRITE|SYNCHRONIZE|FILE_READ_ATTRIBUTES,&atFlash,&ioFlash,0,0,FILE_SHARE_READ,TRUNCATE_EXISTING,FILE_SYNCHRONOUS_IO_NONALERT); //TRUNCATE_EXISTING
		#ifdef DEBUG_MSG
		dprintf("NtCreateFile: %08x\n",sta);
		#endif
		if(sta == 0)
		{
			while(len >= 0x4000)
			{
				sta = NtWriteFile(hFlashFile, 0, 0, 0, &ioFlash, (buffer+dwWritten), 0x4000, 0);
				len = len - 0x4000;
				dwWritten = dwWritten+0x4000;
				#ifdef DEBUG_MSG
				dprintf("NtWriteFile (0x4000): %08x rem: %x\n",sta, len);
				#endif
			}
			if(len != 0)
			{
				sta = NtWriteFile(hFlashFile, 0, 0, 0, &ioFlash, buffer+dwWritten, len, 0);
				dwWritten = dwWritten+len;
				#ifdef DEBUG_MSG
				dprintf("NtWriteFile (0x%x): %08x\n",len,sta);
				#endif
			}
		}
		staNN = NtClose(hFlashFile);
		#ifdef DEBUG_MSG
		dprintf("NtClose hFlashFile: %08x\n",staNN);
		#endif
	}
	if(sta < 0)
	{
		staNN = NtDeviceIoControlFile(hFlashDev, 0, 0, 0, &ioFlash, 0x3c804c, 0, 0, 0, 0); // end sync/async?
		#ifdef DEBUG_MSG
		dprintf("NtDeviceIoControlFile 0x3c804c: %08x\n",staNN);
		#endif
	}
	else
	{
		staNN = NtDeviceIoControlFile(hFlashDev, 0, 0, 0, &ioFlash, 0x3c8048, 0, 0, 0, 0);// end sync/async and flush?
		#ifdef DEBUG_MSG
		dprintf("NtDeviceIoControlFile 0x3c8048: %08x\n",staNN);
		#endif
	}
	staNN = NtClose(hFlashDev);
	#ifdef DEBUG_MSG
	dprintf("NtClose hFlashDev: %08x\n",staNN);
	#endif
	return sta;
}

#define MAX_SIZE_FLASHFILE 0x10000
BOOL FileCopy(char* fileName, char* targetName)
{
	DWORD dwBread, dwFread, sta;
	BOOL res = FALSE;
	BOOL isEqual = FALSE;
	BYTE* filedata = new BYTE[MAX_SIZE_FLASHFILE];
	// read in the file to be flashed
	dwBread = readFileToBuf(fileName, filedata, MAX_SIZE_FLASHFILE);
	if(dwBread != 0)
	{
		// read in the file from flash, if it exists
		char tempName[MAX_PATH] = "Flash:\\";
		strcat_s(tempName, MAX_PATH, targetName);

		BOOL exist = fileExists(tempName);
		if(exist)
		{
			BYTE* targetdata = new BYTE[MAX_SIZE_FLASHFILE];

			dwFread = readFileToBuf(tempName, targetdata, MAX_SIZE_FLASHFILE);
			//dprintf("dwFread = %x, dwBread = %x\n", dwFread, dwBread);
			if(dwFread == dwBread) // identical size, check to see they are the same
			{
				if(byteBufCmp(filedata, targetdata, dwFread))
					isEqual = TRUE;
			}
		}
		//else
		//	dprintf("%s does not exist\n", tempName);

		if(!isEqual)
		{
			sta = writeFileToFlash(filedata, targetName, dwBread);
			if(sta == 0)
			{
				dprintf("Success! (wrote %d bytes to %s)\n", dwBread, targetName);
				res = TRUE;
			}
		}
		else
			dprintf("File is already current in flash\n");
	}
	else
		dprintf("skipped (error %d trying to read file to buffer)\n", GetLastError());
	delete[] filedata;
	return res;
}

void installUpdate(void)
{
	// copy the main DLL to flash
	if(fileExists("GAME:\\files\\launch.xex"))
	{
		dprintf("\nwriting launch.xex to flash... ");
		FileCopy("GAME:\\files\\launch.xex", "launch.xex");
	}
	// copy the helper title xex to flash
	if(fileExists("GAME:\\files\\lhelper.xex"))
	{
		dprintf("writing lhelper.xex to flash... ");
		FileCopy("GAME:\\files\\lhelper.xex", "lhelper.xex");
	}
	// check if any of the ini need updating
	iniUpdate();
	// if after updating ini there is one present in GAME: then write it to flash
	if(fileExists("GAME:\\files\\launch.ini"))
	{
		dprintf("writing launch.ini to flash... ");
		FileCopy("GAME:\\files\\launch.ini", "launch.ini");
	}
	// now lets see about updating patches...
	if(patchExists())
	{
		updatePatches();
	}
	// all done
	dprintf("\n\nFlashing done, Press A to shut down\n");
	for(; ; )
	{
		ATG::Input::GetMergedInput();

		if( ATG::Input::m_DefaultGamepad.wPressedButtons & XINPUT_GAMEPAD_A )
		{
			HalReturnToFirmware(5);
			break;
		}
	}
}

VOID __cdecl main()
{ 
	DWORD flashexist = 0;
	BOOL kernelMatch = FALSE;
   // Initialize the console window
    console.Create( "game:\\Media\\Fonts\\Arial_12.xpr", 0xFF000000, 0xFFFFFFFF );
	console.Clear();

	flashexist = (Mount( "Flash:","\\Device\\Flash") == S_OK);
	if(!flashexist)
	{
		console.Format("ERROR: Couldn't map Flash.\n");
		Sleep(5000);
		XLaunchNewImage(XLAUNCH_KEYWORD_DEFAULT_APP, 0);
	}

	dprintf("\n\n Dash Launch %d.%02d installer for %d\n", VER_MAJ, VER_MIN, TARGET_KERNEL);
	dprintf("-------------------------------------------------------\n\n");
	if(XboxKrnlVersion->Build == TARGET_KERNEL)
	{
		dprintf("Press A to install or update Dash Launch\n");
		kernelMatch = TRUE;
	}
	else
		dprintf("\n\n**Incorrect Kernel Version***\n you should have %d but instead have %d\n", TARGET_KERNEL, XboxKrnlVersion->Build);
	dprintf("Press Y exit\n" );

    for(; ; )
    {
        ATG::Input::GetMergedInput();

		if( ATG::Input::m_DefaultGamepad.wPressedButtons & XINPUT_GAMEPAD_Y )
		{
			XLaunchNewImage(XLAUNCH_KEYWORD_DEFAULT_APP, 0);
			break;
		}
		else if( ATG::Input::m_DefaultGamepad.wPressedButtons & XINPUT_GAMEPAD_A )
		{
			if(kernelMatch)
				installUpdate();
		}
	}
}
